; $Id: VBoxRT-openssl-ose.def $
;; @file
; IPRT - Windows OpenSSL exports.
;
; This file is appended to the architecture specific .def file.
;
; E:\coding\vbox\svn\trunk\tools\win.x86\vcc\v8sp1\bin\x86_amd64\dumpbin.exe \coding\vbox\svn\trunk\out\win.amd64\debug\lib\VBox-libcrypto.lib /symbols /OUT:foo.lst
; grep " External " foo.lst | grep -v " UNDEF " | grep -v " ??_" | grep -v " notype  " | sort   | cut -d "|" -f 2
;

;
; Copyright (C) 2009-2011 Oracle Corporation
;
; This file is part of VirtualBox Open Source Edition (OSE), as
; available from http://www.virtualbox.org. This file is free software;
; you can redistribute it and/or modify it under the terms of the GNU
; General Public License (GPL) as published by the Free Software
; Foundation, in version 2 as it comes in the "COPYING" file of the
; VirtualBox OSE distribution. VirtualBox OSE is distributed in the
; hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
;
; The contents of this file may alternatively be used under the terms
; of the Common Development and Distribution License Version 1.0
; (CDDL) only, as it comes in the "COPYING.CDDL" file of the
; VirtualBox OSE distribution, in which case the provisions of the
; CDDL are applicable instead of those of the GPL.
;
; You may elect to license modified versions of this file under the
; terms and conditions of either the GPL or the CDDL or both.
;

    ;
    ; OpenSSL symbols
    ;
;;    _CONF_get_section
;;    _CONF_get_string
    a2d_ASN1_OBJECT
    a2i_ASN1_INTEGER
    a2i_ASN1_STRING
    a2i_ipadd
    a2i_IPADDRESS
    a2i_IPADDRESS_NC
    ACCESS_DESCRIPTION_free
    ACCESS_DESCRIPTION_new
    asn1_add_error
    ASN1_BIT_STRING_free
    ASN1_BIT_STRING_get_bit
    ASN1_BIT_STRING_new
    ASN1_BIT_STRING_set
    ASN1_BIT_STRING_set_bit
    ASN1_BMPSTRING_free
    ASN1_BMPSTRING_new
    ASN1_check_infinite_end
    ASN1_const_check_infinite_end
    asn1_const_Finish
    ASN1_d2i_bio
    ASN1_d2i_fp
    ASN1_digest
    asn1_do_adb
    asn1_do_lock
    ASN1_dup
    asn1_enc_free
    asn1_enc_init
    asn1_enc_restore
    asn1_enc_save
    ASN1_ENUMERATED_free
    ASN1_ENUMERATED_get
    ASN1_ENUMERATED_new
    ASN1_ENUMERATED_set
    ASN1_ENUMERATED_to_BN
    asn1_ex_c2i
    asn1_ex_i2c
    asn1_Finish
    ASN1_GENERALIZEDTIME_check
    ASN1_GENERALIZEDTIME_free
    ASN1_GENERALIZEDTIME_new
    ASN1_GENERALIZEDTIME_print
    ASN1_GENERALIZEDTIME_set
    ASN1_GENERALIZEDTIME_set_string
    ASN1_GENERALSTRING_free
    ASN1_GENERALSTRING_new
    ASN1_generate_nconf
    ASN1_generate_v3
    asn1_get_choice_selector
    asn1_get_field_ptr
    ASN1_get_object
    asn1_GetSequence
    ASN1_i2d_bio
    ASN1_i2d_fp
    ASN1_IA5STRING_free
    ASN1_IA5STRING_new
    ASN1_INTEGER_cmp
    ASN1_INTEGER_dup
    ASN1_INTEGER_free
    ASN1_INTEGER_get
    ASN1_INTEGER_new
    ASN1_INTEGER_set
    ASN1_INTEGER_to_BN
    ASN1_item_d2i
    ASN1_item_d2i_bio
    ASN1_item_d2i_fp
    ASN1_item_digest
    ASN1_item_dup
    ASN1_item_ex_d2i
    ASN1_item_ex_free
    ASN1_item_ex_i2d
    ASN1_item_ex_new
    ASN1_item_free
    ASN1_item_i2d
    ASN1_item_i2d_bio
    ASN1_item_i2d_fp
    ASN1_item_ndef_i2d
    ASN1_item_new
    ASN1_item_pack
    ASN1_item_sign
    ASN1_item_unpack
    ASN1_item_verify
    ASN1_mbstring_copy
    ASN1_mbstring_ncopy
    ASN1_NULL_free
    ASN1_NULL_new
    ASN1_OBJECT_create
    ASN1_OBJECT_free
    ASN1_OBJECT_new
    ASN1_object_size
    ASN1_OCTET_STRING_cmp
    ASN1_OCTET_STRING_dup
    ASN1_OCTET_STRING_free
    ASN1_OCTET_STRING_new
    ASN1_OCTET_STRING_set
    ASN1_pack_string
    ASN1_parse
    ASN1_parse_dump
    ASN1_primitive_free
    ASN1_primitive_new
    ASN1_PRINTABLE_free
    ASN1_PRINTABLE_new
    ASN1_PRINTABLE_type
    ASN1_PRINTABLESTRING_free
    ASN1_PRINTABLESTRING_new
    ASN1_put_eoc
    ASN1_put_object
    ASN1_seq_pack
    ASN1_seq_unpack
    asn1_set_choice_selector
    ASN1_sign
    ASN1_STRING_cmp
    ASN1_STRING_data
    ASN1_STRING_dup
    ASN1_STRING_free
    ASN1_STRING_get_default_mask
    ASN1_STRING_length
    ASN1_STRING_length_set
    ASN1_STRING_new
    ASN1_STRING_print
    ASN1_STRING_print_ex
    ASN1_STRING_print_ex_fp
    ASN1_STRING_set
    ASN1_STRING_set0
    ASN1_STRING_set_by_NID
    ASN1_STRING_set_default_mask
    ASN1_STRING_set_default_mask_asc
    ASN1_STRING_TABLE_add
    ASN1_STRING_TABLE_cleanup
    ASN1_STRING_TABLE_get
    ASN1_STRING_to_UTF8
    ASN1_STRING_type
    ASN1_STRING_type_new
    ASN1_T61STRING_free
    ASN1_T61STRING_new
    ASN1_tag2bit
    ASN1_tag2str
    ASN1_template_d2i
    ASN1_template_free
    ASN1_template_i2d
    ASN1_template_new
    ASN1_TIME_check
    ASN1_TIME_free
    ASN1_TIME_new
    ASN1_TIME_print
    ASN1_TIME_set
    ASN1_TIME_to_generalizedtime
    ASN1_TYPE_free
    ASN1_TYPE_get
    ASN1_TYPE_get_int_octetstring
    ASN1_TYPE_get_octetstring
    ASN1_TYPE_new
    ASN1_TYPE_set
    ASN1_TYPE_set1
    ASN1_TYPE_set_int_octetstring
    ASN1_TYPE_set_octetstring
    ASN1_UNIVERSALSTRING_free
    ASN1_UNIVERSALSTRING_new
    ASN1_UNIVERSALSTRING_to_string
    ASN1_unpack_string
    ASN1_UTCTIME_check
    ASN1_UTCTIME_cmp_time_t
    ASN1_UTCTIME_free
    ASN1_UTCTIME_new
    ASN1_UTCTIME_print
    ASN1_UTCTIME_set
    ASN1_UTCTIME_set_string
    ASN1_UTF8STRING_free
    ASN1_UTF8STRING_new
    ASN1_verify
    ASN1_VISIBLESTRING_free
    ASN1_VISIBLESTRING_new
    AUTHORITY_INFO_ACCESS_free
    AUTHORITY_INFO_ACCESS_new
    AUTHORITY_KEYID_free
    AUTHORITY_KEYID_new
    BASIC_CONSTRAINTS_free
    BASIC_CONSTRAINTS_new
    BIO_callback_ctrl
    BIO_clear_flags
    BIO_copy_next_retry
    BIO_ctrl
    BIO_ctrl_pending
    BIO_ctrl_wpending
    BIO_dump
    BIO_dump_cb
    BIO_dump_fp
    BIO_dump_indent
    BIO_dump_indent_cb
    BIO_dump_indent_fp
    BIO_dup_chain
    BIO_f_buffer
    BIO_find_type
    BIO_free
    BIO_free_all
    BIO_get_callback
    BIO_get_callback_arg
    BIO_get_ex_data
    BIO_get_ex_new_index
    BIO_get_retry_BIO
    BIO_get_retry_reason
    BIO_gets
    BIO_indent
    BIO_int_ctrl
    BIO_method_name
    BIO_method_type
    BIO_new
    BIO_new_file
    BIO_new_fp
    BIO_new_socket
    BIO_next
    BIO_number_read
    BIO_number_written
    BIO_pop
    BIO_ptr_ctrl
    BIO_push
    BIO_puts
    BIO_read
    BIO_s_file
    BIO_s_socket
    BIO_set
    BIO_set_callback
    BIO_set_callback_arg
    BIO_set_ex_data
    BIO_set_flags
    BIO_test_flags
    BIO_vfree
    BIO_write
    BN_add
    BN_add_word
    bn_add_words
    BN_bin2bn
    BN_BLINDING_convert
    BN_BLINDING_convert_ex
    BN_BLINDING_create_param
    BN_BLINDING_free
    BN_BLINDING_get_flags
    BN_BLINDING_get_thread_id
    BN_BLINDING_invert
    BN_BLINDING_invert_ex
    BN_BLINDING_new
    BN_BLINDING_set_flags
    BN_BLINDING_set_thread_id
    BN_BLINDING_update
    BN_bn2bin
    BN_bn2dec
    BN_bn2hex
    BN_bntest_rand
    BN_clear
    BN_clear_bit
    BN_clear_free
    BN_cmp
    BN_copy
    BN_CTX_end
    BN_CTX_free
    BN_CTX_get
    BN_CTX_init
    BN_CTX_new
    BN_CTX_start
    BN_dec2bn
    BN_div
    BN_div_recp
    BN_div_word
    bn_div_words
    BN_dup
    bn_dup_expand
    BN_exp
    bn_expand2
    BN_free
    BN_from_montgomery
    BN_gcd
    BN_GENCB_call
    BN_generate_prime_ex
    BN_get0_nist_prime_192
    BN_get0_nist_prime_224
    BN_get0_nist_prime_256
    BN_get0_nist_prime_384
    BN_get0_nist_prime_521
    BN_get_params
    BN_get_word
    BN_GF2m_add
    BN_GF2m_arr2poly
    BN_GF2m_mod
    BN_GF2m_mod_arr
    BN_GF2m_mod_div
    BN_GF2m_mod_div_arr
    BN_GF2m_mod_exp
    BN_GF2m_mod_exp_arr
    BN_GF2m_mod_inv
    BN_GF2m_mod_inv_arr
    BN_GF2m_mod_mul
    BN_GF2m_mod_mul_arr
    BN_GF2m_mod_solve_quad
    BN_GF2m_mod_solve_quad_arr
    BN_GF2m_mod_sqr
    BN_GF2m_mod_sqr_arr
    BN_GF2m_mod_sqrt
    BN_GF2m_mod_sqrt_arr
    BN_GF2m_poly2arr
    BN_hex2bn
    BN_init
    BN_is_bit_set
    BN_is_prime_ex
    BN_is_prime_fasttest_ex
    BN_kronecker
    BN_lshift
    BN_lshift1
    BN_mask_bits
    BN_mod_add
    BN_mod_add_quick
    BN_mod_exp
    BN_mod_exp_mont
    BN_mod_exp_mont_consttime
    BN_mod_exp_mont_word
    BN_mod_exp_recp
    BN_mod_exp_simple
    BN_mod_inverse
    BN_mod_lshift
    BN_mod_lshift1
    BN_mod_lshift1_quick
    BN_mod_lshift_quick
    BN_mod_mul
    BN_mod_mul_montgomery
    BN_mod_mul_reciprocal
    BN_mod_sqr
    BN_mod_sqrt
    BN_mod_sub
    BN_mod_sub_quick
    BN_mod_word
    BN_MONT_CTX_copy
    BN_MONT_CTX_free
    BN_MONT_CTX_init
    BN_MONT_CTX_new
    BN_MONT_CTX_set
    BN_MONT_CTX_set_locked
    BN_mul
    bn_mul_add_words
    BN_mul_word
    bn_mul_words
    BN_new
    BN_nist_mod_192
    BN_nist_mod_224
    BN_nist_mod_256
    BN_nist_mod_384
    BN_nist_mod_521
    BN_nnmod
    BN_num_bits
    BN_num_bits_word
    BN_print
    BN_pseudo_rand
    BN_pseudo_rand_range
    BN_rand
    BN_rand_range
    BN_reciprocal
    BN_RECP_CTX_free
    BN_RECP_CTX_init
    BN_RECP_CTX_new
    BN_RECP_CTX_set
    BN_rshift
    BN_rshift1
    BN_set_bit
    BN_set_negative
    BN_set_params
    BN_set_word
    BN_sqr
    bn_sqr_words
    BN_sub
    BN_sub_word
    bn_sub_words
    BN_swap
    BN_to_ASN1_ENUMERATED
    BN_to_ASN1_INTEGER
    BN_uadd
    BN_ucmp
    BN_usub
    BN_value_one
    BUF_MEM_free
    BUF_MEM_grow
    BUF_MEM_grow_clean
    BUF_MEM_new
    BUF_memdup
    BUF_strdup
    BUF_strlcat
    BUF_strlcpy
    BUF_strndup
    c2i_ASN1_BIT_STRING
    c2i_ASN1_INTEGER
    c2i_ASN1_OBJECT
    CERTIFICATEPOLICIES_free
    CERTIFICATEPOLICIES_new
    CONF_dump_bio
    CONF_dump_fp
    CONF_free
    CONF_get_number
    CONF_get_section
    CONF_get_string
    CONF_load
    CONF_load_bio
    CONF_load_fp
    CONF_parse_list
    CONF_set_default_method
    CONF_set_nconf
    CRL_DIST_POINTS_free
    CRL_DIST_POINTS_new
    CRYPTO_add_lock
    CRYPTO_cleanup_all_ex_data
    CRYPTO_dbg_free
    CRYPTO_dbg_get_options
    CRYPTO_dbg_malloc
;;    CRYPTO_dbg_pop_info
;;    CRYPTO_dbg_push_info
    CRYPTO_dbg_realloc
;;    CRYPTO_dbg_remove_all_info
    CRYPTO_dbg_set_options
    CRYPTO_destroy_dynlockid
    CRYPTO_dup_ex_data
    CRYPTO_ex_data_new_class
    CRYPTO_free
    CRYPTO_free_ex_data
    CRYPTO_free_locked
    CRYPTO_get_add_lock_callback
    CRYPTO_get_dynlock_create_callback
    CRYPTO_get_dynlock_destroy_callback
    CRYPTO_get_dynlock_lock_callback
    CRYPTO_get_dynlock_value
    CRYPTO_get_ex_data
    CRYPTO_get_ex_data_implementation
    CRYPTO_get_ex_new_index
    CRYPTO_get_id_callback
    CRYPTO_get_locked_mem_ex_functions
    CRYPTO_get_locked_mem_functions
    CRYPTO_get_locking_callback
    CRYPTO_get_mem_debug_functions
    CRYPTO_get_mem_debug_options
    CRYPTO_get_mem_ex_functions
    CRYPTO_get_mem_functions
    CRYPTO_get_new_dynlockid
    CRYPTO_get_new_lockid
    CRYPTO_is_mem_check_on
    CRYPTO_lock
    CRYPTO_malloc
;;    CRYPTO_malloc_debug_init
    CRYPTO_malloc_locked
    CRYPTO_mem_ctrl
    CRYPTO_mem_leaks
    CRYPTO_mem_leaks_cb
    CRYPTO_mem_leaks_fp
    CRYPTO_new_ex_data
    CRYPTO_num_locks
    CRYPTO_pop_info
    CRYPTO_push_info_
    CRYPTO_realloc
    CRYPTO_realloc_clean
    CRYPTO_remalloc
    CRYPTO_remove_all_info
    CRYPTO_set_add_lock_callback
    CRYPTO_set_dynlock_create_callback
    CRYPTO_set_dynlock_destroy_callback
    CRYPTO_set_dynlock_lock_callback
    CRYPTO_set_ex_data
    CRYPTO_set_ex_data_implementation
    CRYPTO_set_id_callback
    CRYPTO_set_locked_mem_ex_functions
    CRYPTO_set_locked_mem_functions
    CRYPTO_set_locking_callback
    CRYPTO_set_mem_debug_functions
    CRYPTO_set_mem_debug_options
    CRYPTO_set_mem_ex_functions
    CRYPTO_set_mem_functions
;;    CRYPTO_set_mem_info_functions
    CRYPTO_strdup
    CRYPTO_thread_id
    d2i_ACCESS_DESCRIPTION
    d2i_ASN1_BIT_STRING
    d2i_ASN1_BMPSTRING
    d2i_ASN1_BOOLEAN
    d2i_ASN1_bytes
    d2i_ASN1_ENUMERATED
    d2i_ASN1_GENERALIZEDTIME
    d2i_ASN1_GENERALSTRING
    d2i_ASN1_IA5STRING
    d2i_ASN1_INTEGER
    d2i_ASN1_NULL
    d2i_ASN1_OBJECT
    d2i_ASN1_OCTET_STRING
    d2i_ASN1_PRINTABLE
    d2i_ASN1_PRINTABLESTRING
    d2i_ASN1_SET
    d2i_ASN1_T61STRING
    d2i_ASN1_TIME
    d2i_ASN1_TYPE
    d2i_ASN1_type_bytes
    d2i_ASN1_UINTEGER
    d2i_ASN1_UNIVERSALSTRING
    d2i_ASN1_UTCTIME
    d2i_ASN1_UTF8STRING
    d2i_ASN1_VISIBLESTRING
    d2i_AUTHORITY_INFO_ACCESS
    d2i_AUTHORITY_KEYID
    d2i_AutoPrivateKey
    d2i_BASIC_CONSTRAINTS
    d2i_CERTIFICATEPOLICIES
    d2i_CRL_DIST_POINTS
    d2i_DHparams
    d2i_DIRECTORYSTRING
    d2i_DISPLAYTEXT
    d2i_DIST_POINT
    d2i_DIST_POINT_NAME
    d2i_EC_PUBKEY
    d2i_EC_PUBKEY_bio
    d2i_ECDSA_SIG
    d2i_ECParameters
    d2i_ECPKParameters
    d2i_ECPrivateKey
    d2i_ECPrivateKey_bio
    d2i_EDIPARTYNAME
    d2i_EXTENDED_KEY_USAGE
    d2i_GENERAL_NAME
    d2i_GENERAL_NAMES
    d2i_NETSCAPE_SPKAC
    d2i_NETSCAPE_SPKI
    d2i_NOTICEREF
    d2i_OCSP_BASICRESP
    d2i_OCSP_CERTID
    d2i_OCSP_CERTSTATUS
    d2i_OCSP_CRLID
    d2i_OCSP_ONEREQ
    d2i_OCSP_REQINFO
    d2i_OCSP_REQUEST
    d2i_OCSP_RESPBYTES
    d2i_OCSP_RESPDATA
    d2i_OCSP_RESPID
    d2i_OCSP_RESPONSE
    d2i_OCSP_REVOKEDINFO
    d2i_OCSP_SERVICELOC
    d2i_OCSP_SIGNATURE
    d2i_OCSP_SINGLERESP
    d2i_OTHERNAME
    d2i_PKCS7
    d2i_PKCS7_bio
    d2i_PKCS7_DIGEST
    d2i_PKCS7_ENC_CONTENT
    d2i_PKCS7_ENCRYPT
    d2i_PKCS7_ENVELOPE
    d2i_PKCS7_ISSUER_AND_SERIAL
    d2i_PKCS7_RECIP_INFO
    d2i_PKCS7_SIGN_ENVELOPE
    d2i_PKCS7_SIGNED
    d2i_PKCS7_SIGNER_INFO
    d2i_PKCS8_bio
    d2i_PKCS8_PRIV_KEY_INFO
    d2i_PKCS8_PRIV_KEY_INFO_bio
    d2i_PKEY_USAGE_PERIOD
    d2i_POLICYINFO
    d2i_POLICYQUALINFO
    d2i_PrivateKey
    d2i_PrivateKey_bio
    d2i_PROXY_CERT_INFO_EXTENSION
    d2i_PROXY_POLICY
    d2i_PUBKEY
    d2i_PUBKEY_bio
    d2i_PublicKey
    d2i_RSA_PUBKEY
    d2i_RSA_PUBKEY_bio
    d2i_RSAPrivateKey
    d2i_RSAPrivateKey_bio
    d2i_RSAPublicKey
    d2i_RSAPublicKey_bio
    d2i_SXNET
    d2i_SXNETID
    d2i_USERNOTICE
    d2i_X509
    d2i_X509_ALGOR
    d2i_X509_ALGORS
    d2i_X509_ATTRIBUTE
    d2i_X509_AUX
    d2i_X509_bio
    d2i_X509_CERT_AUX
    d2i_X509_CERT_PAIR
    d2i_X509_CINF
    d2i_X509_CRL
    d2i_X509_CRL_bio
    d2i_X509_CRL_INFO
    d2i_X509_EXTENSION
    d2i_X509_EXTENSIONS
    d2i_X509_NAME
    d2i_X509_NAME_ENTRY
    d2i_X509_PUBKEY
    d2i_X509_REQ
    d2i_X509_REQ_bio
    d2i_X509_REQ_INFO
    d2i_X509_REVOKED
    d2i_X509_SIG
    d2i_X509_VAL
    DH_check
    DH_check_pub_key
    DH_compute_key
    DH_free
    DH_generate_key
    DH_get_default_method
    DH_get_ex_data
    DH_get_ex_new_index
    DH_new
    DH_new_method
    DH_OpenSSL
    DH_set_default_method
    DH_set_ex_data
    DH_set_method
    DH_size
    DH_up_ref
    DHparams_print
    DHparams_print_fp
    DIRECTORYSTRING_free
    DIRECTORYSTRING_new
    DISPLAYTEXT_free
    DISPLAYTEXT_new
    DIST_POINT_free
    DIST_POINT_NAME_free
    DIST_POINT_NAME_new
    DIST_POINT_new
    EC_get_builtin_curves
    EC_GF2m_simple_method
    EC_GFp_mont_method
    EC_GFp_nist_method
    EC_GFp_simple_method
    EC_GROUP_check_discriminant
    EC_GROUP_clear_free
    EC_GROUP_cmp
    EC_GROUP_copy
    EC_GROUP_dup
    EC_GROUP_free
    EC_GROUP_get0_generator
    EC_GROUP_get0_seed
    EC_GROUP_get_asn1_flag
    EC_GROUP_get_basis_type
    EC_GROUP_get_cofactor
    EC_GROUP_get_curve_GF2m
    EC_GROUP_get_curve_GFp
    EC_GROUP_get_curve_name
    EC_GROUP_get_degree
    EC_GROUP_get_order
    EC_GROUP_get_pentanomial_basis
    EC_GROUP_get_point_conversion_form
    EC_GROUP_get_seed_len
    EC_GROUP_get_trinomial_basis
    EC_GROUP_have_precompute_mult
    EC_GROUP_method_of
    EC_GROUP_new
    EC_GROUP_new_by_curve_name
    EC_GROUP_new_curve_GF2m
    EC_GROUP_new_curve_GFp
    EC_GROUP_precompute_mult
    EC_GROUP_set_asn1_flag
    EC_GROUP_set_curve_GF2m
    EC_GROUP_set_curve_GFp
    EC_GROUP_set_curve_name
    EC_GROUP_set_generator
    EC_GROUP_set_point_conversion_form
    EC_GROUP_set_seed
    EC_KEY_check_key
    EC_KEY_copy
    EC_KEY_dup
    EC_KEY_free
    EC_KEY_generate_key
    EC_KEY_get0_group
    EC_KEY_get0_private_key
    EC_KEY_get0_public_key
    EC_KEY_get_conv_form
    EC_KEY_get_enc_flags
    EC_KEY_get_key_method_data
    EC_KEY_insert_key_method_data
    EC_KEY_new
    EC_KEY_new_by_curve_name
    EC_KEY_precompute_mult
    EC_KEY_print
    EC_KEY_print_fp
    EC_KEY_set_asn1_flag
    EC_KEY_set_conv_form
    EC_KEY_set_enc_flags
    EC_KEY_set_group
    EC_KEY_set_private_key
    EC_KEY_set_public_key
    EC_KEY_up_ref
    EC_METHOD_get_field_type
    EC_POINT_add
    EC_POINT_bn2point
    EC_POINT_clear_free
    EC_POINT_cmp
    EC_POINT_copy
    EC_POINT_dbl
    EC_POINT_dup
    EC_POINT_free
    EC_POINT_get_affine_coordinates_GF2m
    EC_POINT_get_affine_coordinates_GFp
    EC_POINT_get_Jprojective_coordinates_GFp
    EC_POINT_hex2point
    EC_POINT_invert
    EC_POINT_is_at_infinity
    EC_POINT_is_on_curve
    EC_POINT_make_affine
    EC_POINT_method_of
    EC_POINT_mul
    EC_POINT_new
    EC_POINT_oct2point
    EC_POINT_point2bn
    EC_POINT_point2hex
    EC_POINT_point2oct
    EC_POINT_set_affine_coordinates_GF2m
    EC_POINT_set_affine_coordinates_GFp
    EC_POINT_set_compressed_coordinates_GF2m
    EC_POINT_set_compressed_coordinates_GFp
    EC_POINT_set_Jprojective_coordinates_GFp
    EC_POINT_set_to_infinity
    EC_POINTs_make_affine
    EC_POINTs_mul
    ECDSA_do_sign
    ECDSA_do_sign_ex
    ECDSA_get_default_method
    ECDSA_get_ex_data
    ECDSA_get_ex_new_index
    ECDSA_OpenSSL
    ECDSA_set_default_method
    ECDSA_set_ex_data
    ECDSA_set_method
    ECDSA_SIG_free
    ECDSA_SIG_new
    ECDSA_sign
    ECDSA_sign_ex
    ECDSA_sign_setup
    ECDSA_size
    ECDSA_verify
    ECParameters_print
    ECParameters_print_fp
    ECPKParameters_print
    ECPKParameters_print_fp
    EDIPARTYNAME_free
    EDIPARTYNAME_new
    ERR_add_error_data
    ERR_clear_error
    ERR_error_string
    ERR_error_string_n
    ERR_free_strings
    ERR_func_error_string
    ERR_get_err_state_table
    ERR_get_error
    ERR_get_error_line
    ERR_get_error_line_data
    ERR_get_implementation
    ERR_get_next_error_library
    ERR_get_state
    ERR_get_string_table
    ERR_lib_error_string
    ERR_load_ERR_strings
    ERR_load_strings
    ERR_peek_error
    ERR_peek_error_line
    ERR_peek_error_line_data
    ERR_peek_last_error
    ERR_peek_last_error_line
    ERR_peek_last_error_line_data
    ERR_print_errors
    ERR_print_errors_cb
    ERR_put_error
    ERR_reason_error_string
    ERR_release_err_state_table
    ERR_remove_state
    ERR_set_error_data
    ERR_set_implementation
    ERR_unload_strings
    EVP_add_cipher
    EVP_add_digest
    EVP_aes_128_cbc
    EVP_aes_128_xts
    EVP_aes_192_cbc
    EVP_aes_256_cbc
    EVP_aes_256_xts
    EVP_BytesToKey
    EVP_Cipher
    EVP_CIPHER_asn1_to_param
    EVP_CIPHER_block_size
    EVP_CIPHER_CTX_block_size
    EVP_CIPHER_CTX_cipher
    EVP_CIPHER_CTX_cleanup
    EVP_CIPHER_CTX_clear_flags
    EVP_CIPHER_CTX_ctrl
    EVP_CIPHER_CTX_flags
    EVP_CIPHER_CTX_free
    EVP_CIPHER_CTX_get_app_data
    EVP_CIPHER_CTX_init
    EVP_CIPHER_CTX_iv_length
    EVP_CIPHER_CTX_key_length
    EVP_CIPHER_CTX_new
    EVP_CIPHER_CTX_nid
    EVP_CIPHER_CTX_rand_key
    EVP_CIPHER_CTX_set_app_data
    EVP_CIPHER_CTX_set_flags
    EVP_CIPHER_CTX_set_key_length
    EVP_CIPHER_CTX_set_padding
    EVP_CIPHER_CTX_test_flags
    EVP_CIPHER_flags
    EVP_CIPHER_get_asn1_iv
    EVP_CIPHER_iv_length
    EVP_CIPHER_key_length
    EVP_CIPHER_nid
    EVP_CIPHER_param_to_asn1
    EVP_CIPHER_set_asn1_iv
    EVP_CIPHER_type
    EVP_CipherFinal
    EVP_CipherFinal_ex
    EVP_CipherInit
    EVP_CipherInit_ex
    EVP_CipherUpdate
    EVP_cleanup
    EVP_DecodeBlock
    EVP_DecodeFinal
    EVP_DecodeInit
    EVP_DecodeUpdate
    EVP_DecryptFinal
    EVP_DecryptFinal_ex
    EVP_DecryptInit
    EVP_DecryptInit_ex
    EVP_DecryptUpdate
    EVP_Digest
    EVP_DigestFinal
    EVP_DigestFinal_ex
    EVP_DigestInit
    EVP_DigestInit_ex
    EVP_DigestUpdate
    EVP_ecdsa
    EVP_enc_null
    EVP_EncodeBlock
    EVP_EncodeFinal
    EVP_EncodeInit
    EVP_EncodeUpdate
    EVP_EncryptFinal
    EVP_EncryptFinal_ex
    EVP_EncryptInit
    EVP_EncryptInit_ex
    EVP_EncryptUpdate
    EVP_get_cipherbyname
    EVP_get_digestbyname
    EVP_get_pw_prompt
    EVP_md5
    EVP_MD_block_size
    EVP_MD_CTX_cleanup
    EVP_MD_CTX_clear_flags
    EVP_MD_CTX_copy
    EVP_MD_CTX_copy_ex
    EVP_MD_CTX_create
    EVP_MD_CTX_destroy
    EVP_MD_CTX_init
    EVP_MD_CTX_md
    EVP_MD_CTX_set_flags
    EVP_MD_CTX_test_flags
    EVP_MD_pkey_type
    EVP_MD_size
    EVP_MD_type
    EVP_PBE_alg_add
    EVP_PBE_CipherInit
    EVP_PBE_cleanup
    EVP_PKCS82PKEY
    EVP_PKEY2PKCS8
    EVP_PKEY2PKCS8_broken
    EVP_PKEY_add1_attr
    EVP_PKEY_add1_attr_by_NID
    EVP_PKEY_add1_attr_by_OBJ
    EVP_PKEY_add1_attr_by_txt
    EVP_PKEY_assign
    EVP_PKEY_bits
    EVP_PKEY_cmp
    EVP_PKEY_cmp_parameters
    EVP_PKEY_copy_parameters
    EVP_PKEY_delete_attr
    EVP_PKEY_free
    EVP_PKEY_get1_DH
    EVP_PKEY_get1_EC_KEY
    EVP_PKEY_get1_RSA
    EVP_PKEY_get_attr
    EVP_PKEY_get_attr_by_NID
    EVP_PKEY_get_attr_by_OBJ
    EVP_PKEY_get_attr_count
    EVP_PKEY_missing_parameters
    EVP_PKEY_new
    EVP_PKEY_save_parameters
    EVP_PKEY_set1_DH
    EVP_PKEY_set1_EC_KEY
    EVP_PKEY_set1_RSA
    EVP_PKEY_size
    EVP_PKEY_type
    EVP_rc4
    EVP_read_pw_string
    EVP_set_pw_prompt
    EVP_sha1
    EVP_sha224
    EVP_sha256
    EVP_sha384
    EVP_sha512
    EVP_SignFinal
    EVP_VerifyFinal
    EXTENDED_KEY_USAGE_free
    EXTENDED_KEY_USAGE_new
    GENERAL_NAME_free
    GENERAL_NAME_new
    GENERAL_NAME_print
    GENERAL_NAMES_free
    GENERAL_NAMES_new
    GENERAL_SUBTREE_free
    GENERAL_SUBTREE_new
    hex_to_string
    HMAC
    HMAC_CTX_cleanup
    HMAC_CTX_init
    HMAC_CTX_set_flags
    HMAC_Final
    HMAC_Init
    HMAC_Init_ex
    HMAC_Update
    i2a_ACCESS_DESCRIPTION
    i2a_ASN1_INTEGER
    i2a_ASN1_OBJECT
    i2a_ASN1_STRING
    i2c_ASN1_BIT_STRING
    i2c_ASN1_INTEGER
    i2d_ACCESS_DESCRIPTION
    i2d_ASN1_BIT_STRING
    i2d_ASN1_BMPSTRING
    i2d_ASN1_BOOLEAN
    i2d_ASN1_bytes
    i2d_ASN1_ENUMERATED
    i2d_ASN1_GENERALIZEDTIME
    i2d_ASN1_GENERALSTRING
    i2d_ASN1_IA5STRING
    i2d_ASN1_INTEGER
    i2d_ASN1_NULL
    i2d_ASN1_OBJECT
    i2d_ASN1_OCTET_STRING
    i2d_ASN1_PRINTABLE
    i2d_ASN1_PRINTABLESTRING
    i2d_ASN1_SET
    i2d_ASN1_T61STRING
    i2d_ASN1_TIME
    i2d_ASN1_TYPE
    i2d_ASN1_UNIVERSALSTRING
    i2d_ASN1_UTCTIME
    i2d_ASN1_UTF8STRING
    i2d_ASN1_VISIBLESTRING
    i2d_AUTHORITY_INFO_ACCESS
    i2d_AUTHORITY_KEYID
    i2d_BASIC_CONSTRAINTS
    i2d_CERTIFICATEPOLICIES
    i2d_CRL_DIST_POINTS
    i2d_DHparams
    i2d_DIRECTORYSTRING
    i2d_DISPLAYTEXT
    i2d_DIST_POINT
    i2d_DIST_POINT_NAME
    i2d_EC_PUBKEY
    i2d_EC_PUBKEY_bio
    i2d_ECDSA_SIG
    i2d_ECParameters
    i2d_ECPKParameters
    i2d_ECPrivateKey
    i2d_ECPrivateKey_bio
    i2d_EDIPARTYNAME
    i2d_EXTENDED_KEY_USAGE
    i2d_GENERAL_NAME
    i2d_GENERAL_NAMES
    i2d_NETSCAPE_SPKAC
    i2d_NETSCAPE_SPKI
    i2d_NOTICEREF
    i2d_OCSP_BASICRESP
    i2d_OCSP_CERTID
    i2d_OCSP_CERTSTATUS
    i2d_OCSP_CRLID
    i2d_OCSP_ONEREQ
    i2d_OCSP_REQINFO
    i2d_OCSP_REQUEST
    i2d_OCSP_RESPBYTES
    i2d_OCSP_RESPDATA
    i2d_OCSP_RESPID
    i2d_OCSP_RESPONSE
    i2d_OCSP_REVOKEDINFO
    i2d_OCSP_SERVICELOC
    i2d_OCSP_SIGNATURE
    i2d_OCSP_SINGLERESP
    i2d_OTHERNAME
    i2d_PKCS7
    i2d_PKCS7_bio
    i2d_PKCS7_DIGEST
    i2d_PKCS7_ENC_CONTENT
    i2d_PKCS7_ENCRYPT
    i2d_PKCS7_ENVELOPE
    i2d_PKCS7_ISSUER_AND_SERIAL
    i2d_PKCS7_NDEF
    i2d_PKCS7_RECIP_INFO
    i2d_PKCS7_SIGN_ENVELOPE
    i2d_PKCS7_SIGNED
    i2d_PKCS7_SIGNER_INFO
    i2d_PKCS8_bio
    i2d_PKCS8_PRIV_KEY_INFO
    i2d_PKCS8_PRIV_KEY_INFO_bio
    i2d_PKCS8PrivateKeyInfo_bio
    i2d_PKEY_USAGE_PERIOD
    i2d_POLICYINFO
    i2d_POLICYQUALINFO
    i2d_PrivateKey
    i2d_PrivateKey_bio
    i2d_PROXY_CERT_INFO_EXTENSION
    i2d_PROXY_POLICY
    i2d_PUBKEY
    i2d_PUBKEY_bio
    i2d_PublicKey
    i2d_RSA_PUBKEY
    i2d_RSA_PUBKEY_bio
    i2d_RSAPrivateKey
    i2d_RSAPrivateKey_bio
    i2d_RSAPublicKey
    i2d_RSAPublicKey_bio
    i2d_SXNET
    i2d_SXNETID
    i2d_USERNOTICE
    i2d_X509
    i2d_X509_ALGOR
    i2d_X509_ALGORS
    i2d_X509_ATTRIBUTE
    i2d_X509_AUX
    i2d_X509_bio
    i2d_X509_CERT_AUX
    i2d_X509_CERT_PAIR
    i2d_X509_CINF
    i2d_X509_CRL
    i2d_X509_CRL_bio
    i2d_X509_CRL_INFO
    i2d_X509_EXTENSION
    i2d_X509_EXTENSIONS
    i2d_X509_NAME
    i2d_X509_NAME_ENTRY
    i2d_X509_PUBKEY
    i2d_X509_REQ
    i2d_X509_REQ_bio
    i2d_X509_REQ_INFO
    i2d_X509_REVOKED
    i2d_X509_SIG
    i2d_X509_VAL
    i2o_ECPublicKey
    i2s_ASN1_ENUMERATED
    i2s_ASN1_ENUMERATED_TABLE
    i2s_ASN1_INTEGER
    i2s_ASN1_OCTET_STRING
    i2t_ASN1_OBJECT
    i2v_ASN1_BIT_STRING
    i2v_GENERAL_NAME
    i2v_GENERAL_NAMES
    lh_delete
    lh_doall
    lh_doall_arg
    lh_free
    lh_insert
    lh_new
    lh_num_items
    lh_retrieve
    lh_strhash
    MD5_Final
    MD5_Init
    MD5_Transform
    MD5_Update
    name_cmp
    NAME_CONSTRAINTS_free
    NAME_CONSTRAINTS_new
    NCONF_default
    NCONF_dump_bio
    NCONF_dump_fp
    NCONF_free
    NCONF_free_data
    NCONF_get_number_e
    NCONF_get_section
    NCONF_get_string
    NCONF_load
    NCONF_load_bio
    NCONF_load_fp
    NCONF_new
    NCONF_WIN32
    NETSCAPE_SPKAC_free
    NETSCAPE_SPKAC_new
    NETSCAPE_SPKI_free
    NETSCAPE_SPKI_new
    NETSCAPE_SPKI_sign
    NETSCAPE_SPKI_verify
    NOTICEREF_free
    NOTICEREF_new
    o2i_ECPublicKey
    OBJ_add_object
;;    OBJ_bsearch
;;    OBJ_bsearch_ex
    OBJ_cleanup
    OBJ_cmp
    OBJ_create
    OBJ_create_objects
    OBJ_dup
    OBJ_ln2nid
    OBJ_NAME_add
    OBJ_NAME_cleanup
    OBJ_NAME_do_all
    OBJ_NAME_do_all_sorted
    OBJ_NAME_get
    OBJ_NAME_init
    OBJ_NAME_new_index
    OBJ_NAME_remove
    OBJ_new_nid
    OBJ_nid2ln
    OBJ_nid2obj
    OBJ_nid2sn
    OBJ_obj2nid
    OBJ_obj2txt
    OBJ_sn2nid
    OBJ_txt2nid
    OBJ_txt2obj
    OCSP_BASICRESP_free
    OCSP_BASICRESP_new
    OCSP_CERTID_free
    OCSP_CERTID_new
    OCSP_CERTSTATUS_free
    OCSP_CERTSTATUS_new
    OCSP_CRLID_free
    OCSP_CRLID_new
    OCSP_ONEREQ_free
    OCSP_ONEREQ_new
    OCSP_REQINFO_free
    OCSP_REQINFO_new
    OCSP_REQUEST_free
    OCSP_REQUEST_new
    OCSP_RESPBYTES_free
    OCSP_RESPBYTES_new
    OCSP_RESPDATA_free
    OCSP_RESPDATA_new
    OCSP_RESPID_free
    OCSP_RESPID_new
    OCSP_RESPONSE_free
    OCSP_RESPONSE_new
    OCSP_REVOKEDINFO_free
    OCSP_REVOKEDINFO_new
    OCSP_SERVICELOC_free
    OCSP_SERVICELOC_new
    OCSP_SIGNATURE_free
    OCSP_SIGNATURE_new
    OCSP_SINGLERESP_free
    OCSP_SINGLERESP_new
    OPENSSL_cleanse
    OPENSSL_isservice
    OpenSSLDie
    OTHERNAME_free
    OTHERNAME_new
    PEM_ASN1_read_bio
    PEM_ASN1_write_bio
    PEM_bytes_read_bio
    PEM_def_callback
    PEM_dek_info
    PEM_do_header
    PEM_get_EVP_CIPHER_INFO
    PEM_proc_type
    PEM_read_bio
    PEM_read_bio_PrivateKey
    PEM_read_bio_RSAPrivateKey
    PEM_read_bio_X509
    PEM_read_bio_X509_AUX
    PEM_write_bio
    PEM_write_bio_X509
    PKCS7_DIGEST_free
    PKCS7_DIGEST_new
    PKCS7_dup
    PKCS7_ENC_CONTENT_free
    PKCS7_ENC_CONTENT_new
    PKCS7_ENCRYPT_free
    PKCS7_ENCRYPT_new
    PKCS7_ENVELOPE_free
    PKCS7_ENVELOPE_new
    PKCS7_free
    PKCS7_ISSUER_AND_SERIAL_digest
    PKCS7_ISSUER_AND_SERIAL_free
    PKCS7_ISSUER_AND_SERIAL_new
    PKCS7_new
    PKCS7_RECIP_INFO_free
    PKCS7_RECIP_INFO_new
    PKCS7_SIGN_ENVELOPE_free
    PKCS7_SIGN_ENVELOPE_new
    PKCS7_SIGNED_free
    PKCS7_SIGNED_new
    PKCS7_SIGNER_INFO_free
    PKCS7_SIGNER_INFO_new
    PKCS8_PRIV_KEY_INFO_free
    PKCS8_PRIV_KEY_INFO_new
    PKCS8_set_broken
    PKEY_USAGE_PERIOD_free
    PKEY_USAGE_PERIOD_new
    POLICY_CONSTRAINTS_free
    POLICY_CONSTRAINTS_new
    POLICY_MAPPING_free
    POLICY_MAPPING_new
    POLICYINFO_free
    POLICYINFO_new
    POLICYQUALINFO_free
    POLICYQUALINFO_new
    PROXY_CERT_INFO_EXTENSION_free
    PROXY_CERT_INFO_EXTENSION_new
    PROXY_POLICY_free
    PROXY_POLICY_new
    RAND_add
    RAND_bytes
    RAND_cleanup
    RAND_egd
    RAND_egd_bytes
    RAND_event
    RAND_get_rand_method
    RAND_poll
    RAND_pseudo_bytes
    RAND_query_egd_bytes
    RAND_screen
    RAND_seed
    RAND_set_rand_method
    RAND_SSLeay
    RAND_status
    RC4
    RC4_options
    RC4_set_key
    RSA_blinding_off
    RSA_blinding_on
    RSA_flags
    RSA_free
    RSA_generate_key
    RSA_generate_key_ex
    RSA_get_default_method
    RSA_get_ex_data
    RSA_get_ex_new_index
    RSA_get_method
    RSA_memory_lock
    RSA_new
    RSA_new_method
    RSA_null_method
    RSA_print
    RSA_print_fp
    RSA_private_decrypt
    RSA_private_encrypt
    RSA_public_decrypt
    RSA_public_encrypt
    RSA_set_default_method
    RSA_set_ex_data
    RSA_set_method
    RSA_setup_blinding
    RSA_sign
    RSA_size
    RSA_up_ref
    RSA_verify
;;    RSAPrivateKey_asn1_meth
    RSAPrivateKey_dup
    RSAPublicKey_dup
    s2i_ASN1_INTEGER
    s2i_ASN1_OCTET_STRING
    SHA1_Final
    SHA1_Init
    SHA1_Transform
    SHA1_Update
    SHA224
    SHA224_Final
    SHA224_Init
    SHA224_Update
    SHA256
    SHA256_Final
    SHA256_Init
    SHA256_Transform
    SHA256_Update
    SHA384
    SHA384_Final
    SHA384_Init
    SHA384_Update
    SHA512
    SHA512_Final
    SHA512_Init
    SHA512_Transform
    SHA512_Update
    sk_delete
    sk_delete_ptr
    sk_dup
    sk_find
    sk_find_ex
    sk_free
    sk_insert
    sk_is_sorted
    sk_new
    sk_new_null
    sk_num
    sk_pop
    sk_pop_free
    sk_push
    sk_set
    sk_set_cmp_func
    sk_shift
    sk_sort
    sk_unshift
    sk_value
    sk_zero
    SSLeay
    SSLeay_version
    SSL_accept
    SSL_CTX_free
    SSL_CTX_load_verify_locations
    SSL_CTX_new
    SSL_CTX_set_verify
    SSL_CTX_use_certificate_file
    SSL_CTX_use_PrivateKey_file
    SSL_free
    SSL_get_certificate
    SSL_library_init
    SSL_new
    SSL_pending
    SSL_read
    SSL_set_bio
    SSL_set_read_ahead
    SSL_write
    string_to_hex
    SXNET_add_id_asc
    SXNET_add_id_INTEGER
    SXNET_add_id_ulong
    SXNET_free
    SXNET_get_id_asc
    SXNET_get_id_INTEGER
    SXNET_get_id_ulong
    SXNET_new
    SXNETID_free
    SXNETID_new
    TLSv1_server_method
    UI_add_error_string
    UI_add_info_string
    UI_add_input_boolean
    UI_add_input_string
    UI_add_user_data
    UI_add_verify_string
    UI_construct_prompt
    UI_create_method
    UI_ctrl
    UI_destroy_method
    UI_dup_error_string
    UI_dup_info_string
    UI_dup_input_boolean
    UI_dup_input_string
    UI_dup_verify_string
    UI_free
    UI_get0_action_string
    UI_get0_output_string
    UI_get0_result
    UI_get0_result_string
    UI_get0_test_string
    UI_get0_user_data
    UI_get_default_method
    UI_get_ex_data
    UI_get_ex_new_index
    UI_get_input_flags
    UI_get_method
    UI_get_result_maxsize
    UI_get_result_minsize
    UI_get_string_type
    UI_method_get_closer
    UI_method_get_flusher
    UI_method_get_opener
    UI_method_get_reader
    UI_method_get_writer
    UI_method_set_closer
    UI_method_set_flusher
    UI_method_set_opener
    UI_method_set_reader
    UI_method_set_writer
    UI_new
    UI_new_method
    UI_OpenSSL
    UI_process
    UI_set_default_method
    UI_set_ex_data
    UI_set_method
    UI_set_result
    USERNOTICE_free
    USERNOTICE_new
    UTF8_getc
    UTF8_putc
    v2i_ASN1_BIT_STRING
    v2i_GENERAL_NAME
    v2i_GENERAL_NAME_ex
    v2i_GENERAL_NAMES
    X509_add1_ext_i2d
    X509_add1_reject_object
    X509_add1_trust_object
    X509_add_ext
    X509_ALGOR_dup
    X509_ALGOR_free
    X509_ALGOR_get0
    X509_ALGOR_new
    X509_ALGOR_set0
    X509_alias_get0
    X509_alias_set1
;;    X509_asn1_meth
    X509_ATTRIBUTE_count
    X509_ATTRIBUTE_create
    X509_ATTRIBUTE_create_by_NID
    X509_ATTRIBUTE_create_by_OBJ
    X509_ATTRIBUTE_create_by_txt
    X509_ATTRIBUTE_dup
    X509_ATTRIBUTE_free
    X509_ATTRIBUTE_get0_data
    X509_ATTRIBUTE_get0_object
    X509_ATTRIBUTE_get0_type
    X509_ATTRIBUTE_new
    X509_ATTRIBUTE_set1_data
    X509_ATTRIBUTE_set1_object
    X509_CERT_AUX_free
    X509_CERT_AUX_new
    X509_CERT_AUX_print
    X509_CERT_PAIR_free
    X509_CERT_PAIR_new
    X509_certificate_type
    X509_check_ca
    X509_check_issued
    X509_check_private_key
    X509_check_purpose
    X509_CINF_free
    X509_CINF_new
    X509_cmp
    X509_CRL_add0_revoked
    X509_CRL_add1_ext_i2d
    X509_CRL_add_ext
    X509_CRL_cmp
    X509_CRL_delete_ext
    X509_CRL_digest
    X509_CRL_dup
    X509_CRL_free
    X509_CRL_get_ext
    X509_CRL_get_ext_by_critical
    X509_CRL_get_ext_by_NID
    X509_CRL_get_ext_by_OBJ
    X509_CRL_get_ext_count
    X509_CRL_get_ext_d2i
    X509_CRL_INFO_free
    X509_CRL_INFO_new
    X509_CRL_new
    X509_CRL_sign
    X509_CRL_verify
    X509_delete_ext
    X509_digest
    X509_dup
    X509_email_free
    X509_EXTENSION_create_by_NID
    X509_EXTENSION_create_by_OBJ
    X509_EXTENSION_dup
    X509_EXTENSION_free
    X509_EXTENSION_get_critical
    X509_EXTENSION_get_data
    X509_EXTENSION_get_object
    X509_EXTENSION_new
    X509_EXTENSION_set_critical
    X509_EXTENSION_set_data
    X509_EXTENSION_set_object
    X509_find_by_issuer_and_serial
    X509_find_by_subject
    X509_free
    X509_get0_pubkey_bitstr
    X509_get1_email
    X509_get1_ocsp
    X509_get_ex_data
    X509_get_ex_new_index
    X509_get_ext
    X509_get_ext_by_critical
    X509_get_ext_by_NID
    X509_get_ext_by_OBJ
    X509_get_ext_count
    X509_get_ext_d2i
    X509_get_issuer_name
    X509_get_pubkey
    X509_get_serialNumber
    X509_get_subject_name
    X509_issuer_and_serial_cmp
    X509_issuer_and_serial_hash
    X509_issuer_name_cmp
    X509_issuer_name_hash
    X509_keyid_get0
    X509_keyid_set1
    X509_NAME_add_entry
    X509_NAME_add_entry_by_NID
    X509_NAME_add_entry_by_OBJ
    X509_NAME_add_entry_by_txt
    X509_NAME_cmp
    X509_NAME_delete_entry
    X509_NAME_digest
    X509_NAME_dup
    X509_NAME_entry_count
    X509_NAME_ENTRY_create_by_NID
    X509_NAME_ENTRY_create_by_OBJ
    X509_NAME_ENTRY_create_by_txt
    X509_NAME_ENTRY_dup
    X509_NAME_ENTRY_free
    X509_NAME_ENTRY_get_data
    X509_NAME_ENTRY_get_object
    X509_NAME_ENTRY_new
    X509_NAME_ENTRY_set_data
    X509_NAME_ENTRY_set_object
    X509_NAME_free
    X509_NAME_get_entry
    X509_NAME_get_index_by_NID
    X509_NAME_get_index_by_OBJ
    X509_NAME_get_text_by_NID
    X509_NAME_get_text_by_OBJ
    X509_NAME_hash
    X509_NAME_new
    X509_NAME_oneline
    X509_NAME_print
    X509_NAME_print_ex
    X509_NAME_print_ex_fp
    X509_NAME_set
    X509_new
    X509_ocspid_print
    X509_POLICY_NODE_print
    X509_print
    X509_print_ex
    X509_print_ex_fp
    X509_print_fp
    X509_pubkey_digest
    X509_PUBKEY_free
    X509_PUBKEY_get
    X509_PUBKEY_new
    X509_PUBKEY_set
    X509_PURPOSE_add
    X509_PURPOSE_cleanup
    X509_PURPOSE_get0
    X509_PURPOSE_get0_name
    X509_PURPOSE_get0_sname
    X509_PURPOSE_get_by_id
    X509_PURPOSE_get_by_sname
    X509_PURPOSE_get_count
    X509_PURPOSE_get_id
    X509_PURPOSE_get_trust
    X509_PURPOSE_set
    X509_reject_clear
    X509_REQ_add1_attr
    X509_REQ_add1_attr_by_NID
    X509_REQ_add1_attr_by_OBJ
    X509_REQ_add1_attr_by_txt
    X509_REQ_add_extensions
    X509_REQ_add_extensions_nid
    X509_REQ_check_private_key
    X509_REQ_delete_attr
    X509_REQ_digest
    X509_REQ_dup
    X509_REQ_extension_nid
    X509_REQ_free
    X509_REQ_get1_email
    X509_REQ_get_attr
    X509_REQ_get_attr_by_NID
    X509_REQ_get_attr_by_OBJ
    X509_REQ_get_attr_count
    X509_REQ_get_extension_nids
    X509_REQ_get_extensions
    X509_REQ_get_pubkey
    X509_REQ_INFO_free
    X509_REQ_INFO_new
    X509_REQ_new
    X509_REQ_set_extension_nids
    X509_REQ_set_pubkey
    X509_REQ_set_subject_name
    X509_REQ_set_version
    X509_REQ_sign
    X509_REQ_verify
    X509_REVOKED_add1_ext_i2d
    X509_REVOKED_add_ext
    X509_REVOKED_delete_ext
    X509_REVOKED_free
    X509_REVOKED_get_ext
    X509_REVOKED_get_ext_by_critical
    X509_REVOKED_get_ext_by_NID
    X509_REVOKED_get_ext_by_OBJ
    X509_REVOKED_get_ext_count
    X509_REVOKED_get_ext_d2i
    X509_REVOKED_new
    X509_set_ex_data
    X509_SIG_free
    X509_SIG_new
    X509_sign
    X509_signature_print
    X509_STORE_CTX_cleanup
    X509_STORE_CTX_get_ex_new_index
    X509_STORE_CTX_get0_param
    X509_STORE_CTX_init
    X509_STORE_CTX_set_default
    X509_STORE_CTX_set_ex_data
    X509_STORE_CTX_set_verify_cb
    X509_STORE_free
    X509_STORE_load_locations
    X509_STORE_new
    X509_STORE_set_default_paths
    X509_subject_name_cmp
    X509_subject_name_hash
    X509_supported_extension
    X509_to_X509_REQ
    X509_trust_clear
    X509_VAL_free
    X509_VAL_new
    X509_verify
    X509_verify_cert
    X509_VERIFY_PARAM_free
    X509_VERIFY_PARAM_get_depth
    X509_VERIFY_PARAM_inherit
    X509_VERIFY_PARAM_new
    X509_VERIFY_PARAM_set_depth
    X509_VERIFY_PARAM_set_purpose
    X509_VERIFY_PARAM_set_trust
    X509_VERIFY_PARAM_set1
    X509at_add1_attr
    X509at_add1_attr_by_NID
    X509at_add1_attr_by_OBJ
    X509at_add1_attr_by_txt
    X509at_delete_attr
    X509at_get0_data_by_OBJ
    X509at_get_attr
    X509at_get_attr_by_NID
    X509at_get_attr_by_OBJ
    X509at_get_attr_count
    X509V3_add1_i2d
    X509v3_add_ext
    X509V3_add_standard_extensions
    X509V3_add_value
    X509V3_add_value_bool
    X509V3_add_value_bool_nf
    X509V3_add_value_int
    X509V3_add_value_uchar
    X509V3_conf_free
    X509v3_delete_ext
    X509V3_EXT_add
    X509V3_EXT_add_alias
    X509V3_EXT_add_conf
    X509V3_EXT_add_list
    X509V3_EXT_add_nconf
    X509V3_EXT_add_nconf_sk
    X509V3_EXT_cleanup
    X509V3_EXT_conf
    X509V3_EXT_conf_nid
    X509V3_EXT_CRL_add_conf
    X509V3_EXT_CRL_add_nconf
    X509V3_EXT_d2i
    X509V3_EXT_get
    X509V3_EXT_get_nid
    X509V3_EXT_i2d
    X509V3_EXT_nconf
    X509V3_EXT_nconf_nid
    X509V3_EXT_print
    X509V3_EXT_print_fp
    X509V3_EXT_REQ_add_conf
    X509V3_EXT_REQ_add_nconf
    X509V3_EXT_val_prn
    X509V3_extensions_print
    X509V3_get_d2i
    X509v3_get_ext
    X509v3_get_ext_by_critical
    X509v3_get_ext_by_NID
    X509v3_get_ext_by_OBJ
    X509v3_get_ext_count
    X509V3_get_section
    X509V3_get_string
    X509V3_get_value_bool
    X509V3_get_value_int
    X509V3_NAME_from_section
    X509V3_parse_list
    X509V3_section_free
    X509V3_set_conf_lhash
    X509V3_set_ctx
    X509V3_set_nconf
    X509V3_string_free

