/* $Id: libXcomposite.c $ */

/** @file
 * X.Org libXcomposite.so linker stub
 */

/*
 * Copyright (C) 2010 Oracle Corporation
 *
 * This file is part of VirtualBox Open Source Edition (OSE), as
 * available from http://www.virtualbox.org. This file is free software;
 * you can redistribute it and/or modify it under the terms of the GNU
 * General Public License (GPL) as published by the Free Software
 * Foundation, in version 2 as it comes in the "COPYING" file of the
 * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
 */

#include <iprt/cdefs.h>
#include <iprt/types.h>

DECLEXPORT(void) XCompositeCreateRegionFromBorderClip(void);
void XCompositeCreateRegionFromBorderClip(void) {}
DECLEXPORT(void) XCompositeExtensionInfo(void);
void XCompositeExtensionInfo(void) {}
DECLEXPORT(void) XCompositeExtensionName(void);
void XCompositeExtensionName(void) {}
DECLEXPORT(void) XCompositeFindDisplay(void);
void XCompositeFindDisplay(void) {}
DECLEXPORT(void) XCompositeGetOverlayWindow(void);
void XCompositeGetOverlayWindow(void) {}
DECLEXPORT(void) XCompositeNameWindowPixmap(void);
void XCompositeNameWindowPixmap(void) {}
DECLEXPORT(void) XCompositeQueryExtension(void);
void XCompositeQueryExtension(void) {}
DECLEXPORT(void) XCompositeQueryVersion(void);
void XCompositeQueryVersion(void) {}
DECLEXPORT(void) XCompositeRedirectSubwindows(void);
void XCompositeRedirectSubwindows(void) {}
DECLEXPORT(void) XCompositeRedirectWindow(void);
void XCompositeRedirectWindow(void) {}
DECLEXPORT(void) XCompositeReleaseOverlayWindow(void);
void XCompositeReleaseOverlayWindow(void) {}
DECLEXPORT(void) XCompositeUnredirectSubwindows(void);
void XCompositeUnredirectSubwindows(void) {}
DECLEXPORT(void) XCompositeUnredirectWindow(void);
void XCompositeUnredirectWindow(void) {}
DECLEXPORT(void) XCompositeVersion(void);
void XCompositeVersion(void) {}

