/** @file
  GUID for SecurityPkg PCD Token Space.

Copyright (c) 2009 - 2010, Intel Corporation. All rights reserved.<BR>
This program and the accompanying materials 
are licensed and made available under the terms and conditions of the BSD License 
which accompanies this distribution.  The full text of the license may be found at 
http://opensource.org/licenses/bsd-license.php

THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS, 
WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.

**/

#ifndef _SECURITYPKG_TOKEN_SPACE_GUID_H_
#define _SECURITYPKG_TOKEN_SPACE_GUID_H_

#define SECURITYPKG_TOKEN_SPACE_GUID \
  { \
    0xd3fb176, 0x9569, 0x4d51, { 0xa3, 0xef, 0x7d, 0x61, 0xc6, 0x4f, 0xea, 0xba } \
  }

extern EFI_GUID gEfiSecurityPkgTokenSpaceGuid;

#endif
