/* $Id: VBoxNetFltNobjRc.h $ */
/** @file
 * VBoxNetFltNobjRc.h - Notify Object for Bridged Networking Driver.
 * Resource definitions
 */
/*
 * Copyright (C) 2011 Oracle Corporation
 *
 * This file is part of VirtualBox Open Source Edition (OSE), as
 * available from http://www.virtualbox.org. This file is free software;
 * you can redistribute it and/or modify it under the terms of the GNU
 * General Public License (GPL) as published by the Free Software
 * Foundation, in version 2 as it comes in the "COPYING" file of the
 * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
 */
#ifndef ___VBoxNetFltNobjRc_h___
#define ___VBoxNetFltNobjRc_h___

/* registry script rc ID */
#define IDR_VBOXNETFLT_NOBJ 101

#endif /* #ifndef ___VBoxNetFltNobjRc_h___ */
